#!/bin/bash

################ CONSTANT VALUES ################################################################################
														#
MAXLDEVS=10 						# RECOMMENDED VALUE FOR SIMULTANEOUS ZPR		#
MAX_ZPR_CHECKS=60 					# WILL CHECK THIS MANY TIMES FOR RECLAIMED BLOCKS	#
RAIDCOM="/cygdrive/C/HORCM/etc/raidcom" 		# RAIDCOM LOCATION					#
RAIDCFG="/cygdrive/C/HORCM/etc/raidcfg"
HORCM="/cygdrive/C/HORCM/etc" 				# LOCATION OF HORCM ETC					#
HORCMROOT="/cygdrive/C/HORCM"				# LOCATION OF HORCM					#
ON_CYGWIN=1						# INDICATES THAT RUNNING ON CYGWIN			#
HOSTNAME=$(hostname)					# HOSTNAME						#
REALUSER=$(whoami)					# USER RUNNING SCRIPT					#
VERSION=$($RAIDCOM -h | grep Ver | cut -d: -f2)		# RAIDCOM VERSION					#
HORCMINSTANCES="97" 					# HAS TO BE SET FOR RUNNING UNDER CYGWIN		#
TIMEOUT_RC=60						# TIMEOUT FOR OBTAINING RAIDCOM LOCK			#
REPORT_DIR="./zpr_reports"				# DEFAULT REPORTS DIRECTORY				#	
DEBUG=0							# SET TO  0 TO RUN IN SIMULATION MODE			#
CUT_OFF_THRESHOLD=20					# LDEVs WITH %USED BELOW THIS WILL NOT BE PROCESSED     #
VALID_LDEV_STATES=('NML' 'BSY' 'BLK')										#
VALID_OPE_VALUES=('NONE' 'ZPD' 'RLC' 'RBL' 'FMT' 'QFMT' 'SHRPL' 'SHRD' 'CCOPY' 'CACCS')				#
VALID_LDEV_TYPES=('OPEN-V' 'OPEN-V-CVS' 'OPEN-V-CVS-CM' 'DEFINING' 'REMOVING')					#	
VALID_VOL_ATTR=('CMD' 'CLUN' 'CVS' 'LUSE' 'ALUN' 'ELUN' 'OLG' 'VVOL' 'HORC' 'MRCF' 'QS' 'JNL' 'HDP' 'HDT'\	
		 'POOL' 'QRD' 'ENCD' 'SYSD' 'TSE')								
SKIP_VOL_ATTR=('CMD' 'CLUN' 'LUSE' 'ALUN' 'ELUN' 'OLG' 'VVOL' 'QS' 'JNL' 'HORC' 'MRCF'\				
		 'POOL' 'QRD' 'ENCD' 'SYSD' 'TSE')			
SKIP_VOL_OPE_TYPE=('BSY' 'BLK' 'RLC' 'RBL' 'FMT' 'QFMT' 'SHRPL' 'SHRD' 'CCOPY' 'CACCS' 'OPEN-V-CVS-CM'\
			'DEFINING' 'REMOVING')	
#################################################################################################################

############### INITIALIZE VARIABLES ############################################################################
														#
LDEVCOUNT=1						# COMMAND LINE PARAMETER ; LDEVS TO RUN ZPR AT A TIME	#
SKIPPED_CNT=0						# COUNTER TRACKS SKIPPED LDEVS				#
PROCESSED_CNT=0						# COUNTER TRACKS PROCESSED LDEVS			#
WAIT=1							# COMMAND LINE PARAMETER ; 0 = DO NOT MONITOR STATUS	#
LDEVFILE=""						# FILE CONTAINING LDEV LIST				#
LDEVLIST=""						# COMMAND LINE PARAMETER LIST OF LDEVS			#
CURR_LDEV=""						# GLOBAL VARIABLE ; CURRENT LDEV BEING PROCESSED	#
SERIAL=0						# COMMAND LINE PARAMETER ; STORES SERIAL NUMBER		#
HORCMINST=""						# HORCMINSTANCE DETECTED FOR SERIAL SPECIFIED		#
VERBOSE=1						# OUTPUTS MORE INFORMATION				#
SKIPPED=1						# FLAG INDICATING THAT LDEV WAS SKIPPED			#
RETRIES=1						# TIME TO RETRY SKIPPED LDEVS				#
RAIDUSER=""													#
LOCK_OWNER=""													#
LOCK_HOST=""													#
START_TIME=`date +%m%d%y%0k%0M`											#
END_TIME=""													#
############### ASSOCIATIVE ARRAYS FOR REPORTING ################################################################

declare -A START_LDEV_USED_CAPACITY
declare -A LDEV_TOTAL_CAPACITY
declare -A END_LDEV_USED_CAPACITY
declare -A COUNT_LDEV_ZPR_CHECKS
declare -A SKIPPED_LDEVS_LOCK
declare -A SKIPPED_LDEVS_STATUS
declare -A SKIPPED_LDEVS_INVALID
declare -A PROCESSED_LDEVS
declare -A REPORT_GB_STARTUSED_LDEV
declare -A REPORT_GB_ENDUSED_LDEV
declare -A REPORT_GB_PERCENT_LDEV
declare -a MASTER_LDEV_LIST
declare -a CURRENT_LDEV_LIST
declare -a BAD_FORMAT_LDEV_LIST
declare -a CUT_OFF_LDEVS
declare -A TEMP_LDEV_LIST

############### FUNCTION TO PRINT USAGE HELP #######################################################

usage(){
   [ -n "$1" ] && echo && echo "Error : $1" 
   echo
   echo "Usage : /do_zpr -w -s <SERIAL> -c [1..$MAXLDEVS] -f <filename> | -l ldev1,ldev2,ldev3,.. "
   echo "Zero Page Reclaim Utility for USPV and VSP Dynamic Provisioning Volumes"
   echo
   echo "        -w,  Will run ZPR in a synchronous fashion i.e will wait for X LDEVs to complete"
   echo
   echo "        -c,  Will use the number specified after -c and will run ZPR on that many ldevs simultaneously"
   echo
   echo "        -f,  Specify the filename from which ldevs are to be used"
   echo 
   echo "        -s,  Use to specify array serial number to use"
   echo 
   echo "        -l,  Use to specify ldevs at the command line instead of from a file"
   echo 
   echo "        -v,  Print more information on the console"
   echo ; echo
   echo "Examples :" ; echo
   echo "	do_zpr -s 65536 -c 1 -l 00:01,02:03,04:05"
   echo	" 		-- Run ZPR on ldevs 00:01,02:03,04:05 on array 65536 one at a time and quit" 
   echo
   echo "	do_zpr -w -s 65536 -c 1 -l 00:01,02:03,04:05"
   echo "	 	-- Run ZPR on ldevs 00:01,02:03,04:05 on array 65536 one at a time and monitor status" 
   echo
   echo "	do_zpr -w -s 65536 -c 5 -f ldevs.fil"
   echo " 		-- Run ZPR on ldevs listed in file ldevs.fil on array 65536 five at a time and monitor status" 
   exit 1
}


############### FUNCTION TO CHECK LDEV STATUS ######################################################

check_for_NML(){
	local status=""
	for state in "${VALID_LDEV_STATES[@]}"
	do 
		[[ -z "$status" ]] && $RAIDCOM get ldev -ldev_id $1 -check_status $state 2> /dev/null
		[[ $? -eq 0 ]] && status=$state
	done
	echo $status 
}

############### FUNCTION TO CHECK LDEV OPERATION ###################################################

check_for_OPE(){
	local status=""
	for state in "${VALID_OPE_VALUES[@]}"
	do 
		[[ -z "$status" ]]  && $RAIDCOM get ldev -ldev_id $1 -check_status $state 2> /dev/null
		[[ $? -eq 0 ]] && status=$state
	done
	echo $status 
}

############### FUNCTION TO CHECK FOR DPVOL ########################################################

check_for_TYPE(){
	status=""
	for state in "${VALID_LDEV_TYPES[@]}"
	do 
		[[ -z $status ]]  && $RAIDCOM get ldev -ldev_id $1 -check_status $state 2> /dev/null 
		[[ $? -eq 0 ]] && status=$state
	done
	[[  -z "$status" ]] && status="UNDEFINED"
	echo $status 
}

check_for_ATTR(){
	status=""
	
	ATTR_LIST=$($RAIDCOM get ldev -ldev_id $1 | grep "VOL_ATTR" | sed 's/VOL_ATTR//' | sed 's/:/\,/g' | sed 's/ //g' | sed 's/^\,//')
	for attr in $(printf "$ATTR_LIST" | sed 's/\,/ /g')
	do
		[ -z "$status" ] && check_list_item $attr ${SKIP_VOL_ATTR[@]}
		[[ ( $? -eq 0 ) && ( -z $status ) ]] && status=$ATTR_LIST 
	done
	[ -z "$status" ] && echo "HDP"  || echo $status
}	

####################################################################################################
############### FUNCTION TO RUN ZPR ON LDEV(S)######################################################
####################################################################################################

do_zpr_ldev(){

	(( PROCESSED = 0 ))
	(( SKIPPED = 0 ))

	IS_LOCKED=$($RAIDCOM get resource | grep -v RS_GROUP | tr -s " " | cut -d" " -f3 | tr -d "\n")
	LOCK_OWNER=$($RAIDCOM get resource | grep -v RS_GROUP | tr -s " " | cut -d" " -f4 | tr -d "\n")
	LOCK_HOST=$($RAIDCOM get resource | grep -v RS_GROUP | tr -s " " | cut -d" " -f5 | tr -d "\n")

#IS_LOCKED="Locked"
#LOCK_OWNER="ghostuser"
#LOCK_HOST="debugger"

	[ "$IS_LOCKED" == "Locked" ] && printf "\t\t$SERIAL locked by $LOCK_OWNER on $LOCK_HOST\n"

	$RAIDCOM lock resource -resource_name meta_resource -time $TIMEOUT_RC 2> /dev/null

	IS_LOCKED=$($RAIDCOM get resource | grep -v RS_GROUP | tr -s " " | cut -d" " -f3 | tr -d "\n")
	LOCK_OWNER=$($RAIDCOM get resource | grep -v RS_GROUP | tr -s " " | cut -d" " -f4 | tr -d "\n")
	LOCK_HOST=$($RAIDCOM get resource | grep -v RS_GROUP | tr -s " " | cut -d" " -f5 | tr -d "\n")
	
#IS_LOCKED="Locked"
#LOCK_OWNER="ghostuser"
#LOCK_HOST="debugger"

	for ldev in ${@:1}
	do
		if [[ ( "$IS_LOCKED" == "Locked" ) && ( "$LOCK_OWNER" != "$RAIDUSER" ) ]]
		then
			 printf "\t\tSkipping LDEV $ldev. Reason : Unable to obtain lock on $SERIAL \n"
			 SKIPPED_LDEVS_LOCK[$ldev]=$LOCK_OWNER 
			 (( PROCESSED_LDEVS[$ldev]=0 )) 
			 (( SKIPPED = $SKIPPED + 1 )) 
			 (( PROCESSED = $PROCESSED + 1 )) 
		fi

		[[ $PROCESSED -eq 0 ]] && printf "\t\tStarted ZPR on "	
	
		if [[ ( "$IS_LOCKED" == "Locked" ) && ( "$LOCK_OWNER" == "$RAIDUSER" ) ]]
		then
			 $RAIDCOM modify ldev -ldev_id $ldev -status discard_zero_page 3> /dev/null && printf "$ldev " 
			 (( PROCESSED = $PROCESSED + 1 ))
		fi

		[ $VERBOSE -eq 0 ] && $RAIDCOM get command_status	

		STATUS=$($RAIDCOM get command_status | grep -v "SSB" | tr -s " " | cut -d" " -f2 2> /dev/null)
		REASON=$($RAIDCOM get command_status | grep -v "SSB" | tr -s " " | cut -d" " -f6 2> /dev/null)
		
       		if [[ "$STATUS" == "-" ]]
 		then
           		$RAIDCOM reset command_status
        	else
           		printf $REASON 
           		$RAIDCOM reset command_status
			STATUS=""
        	fi
	
		[[ $VERBOSE -eq 0 ]] && printf "$ldev : TOTAL BLKS ${LDEV_TOTAL_CAPACITY[$ldev]} : USED BLKS ${START_LDEV_USED_CAPACITY[$ldev]}"
	done

	[[ ( "$IS_LOCKED" == "Locked" ) && ( "$LOCK_OWNER" == "$RAIDUSER" ) ]]\
		&& $RAIDCOM unlock resource -resource_name meta_resource -time $TIMEOUT_RC 2> /dev/null

	return $SKIPPED
}


############### SUPPORTING FUNCTION FOR do_zpr_ldev  ###############################################

check_list_item(){
  for item in "${@:2}"; do [[ "$item" == "$1" ]] && return 0; done
  return 1
}

############### FUNCTION TO WAIT UNTIL ZPR IS COMPLETE #############################################

do_ldev_zpr_wait(){
	(( PROCESSED = 0 ))
	for ldev in ${@:1}
	do
		(( COUNT_LDEV_ZPR_CHECKS[$ldev]=0 ))
		(( START_LDEV_USED_CAPACITY[$ldev]=$(get_used_blocks $ldev) ))
        done	
	
	printf "\n"

	while [[ ( $PROCESSED -lt $LDEVCOUNT ) ]]
	do
	     printf "\t\t" 


	     for ldev in ${@:1}
	     do
		if [[ ( ${PROCESSED_LDEVS[$ldev]} -ne 0 ) && ( -n $ldev ) ]]
		then
			if [[ -z ${SKIPPED_LDEVS_LOCK[$ldev]} ]]
			then
			
				if [[ ( "$(check_for_OPE $ldev)" == "ZPD" ) ]]
				then
					COUNT=${COUNT_LDEV_ZPR_CHECKS[$ldev]}
					ENDC=${END_LDEV_USED_CAPACITY[$ldev]}
					TCAP=${LDEV_TOTAL_CAPACITY[$ldev]}
					PERCENT=$(get_percent_used $ENDC $TCAP)

					printf "%4d:%6s:%4d%%"  $COUNT $ldev $PERCENT 

					(( END_LDEV_USED_CAPACITY[$ldev]=$(get_used_blocks $ldev) ))
	
					[[ ( ${START_LDEV_USED_CAPACITY[$ldev]} -eq ${END_LDEV_USED_CAPACITY[$ldev]} ) ]] &&\
					 	(( COUNT_LDEV_ZPR_CHECKS[$ldev]=${COUNT_LDEV_ZPR_CHECKS[$ldev]} + 1 ))
				
					if [[ (  ${COUNT_LDEV_ZPR_CHECKS[$ldev]} -eq $MAX_ZPR_CHECKS ) ]]
					then
						 printf "%4d:%6s:Tmout" ${COUNT_LDEV_ZPR_CHECKS[$ldev]} $ldev
       		 	                         (( PROCESSED_LDEVS[$ldev]=0 ))
       		         	                 (( PROCESSED=$PROCESSED + 1 ))
					fi
				fi	
				if [[ ( "$(check_for_OPE $ldev)" == "NONE" ) ]]
				then
					printf "%4d:%6s:Ended" ${COUNT_LDEV_ZPR_CHECKS[$ldev]} $ldev
       		 	                (( PROCESSED_LDEVS[$ldev]=0 ))
					(( PROCESSED=$PROCESSED + 1 ))				
				fi
			fi	
		else
			printf "%4d:%6s:%5s" ${COUNT_LDEV_ZPR_CHECKS[$ldev]} $ldev $(check_for_OPE $ldev)
		fi
	      done
	      printf "\n" 
	done
	
	return $PROCESSED
}

####################################################################################################
############### FUNCTION TO ASSIST IN LOGGING INTO A RAID ARRAY ####################################
####################################################################################################
do_login(){
        $RAIDCFG -logint
        LOGGEDIN=$?
        [[ $LOGGEDIN -eq 0 ]] && $RAIDCOM -logout && LOGGEDIN=1
        until [[ $LOGGEDIN -eq 0 ]]
        do
                echo "Enter Credentials for RAID Array $SERIAL"
                read -p "Username: " RAIDUSER
                read -s -p "Password: " RAIDPASS; echo ""
                LOGINSTR="$RAIDUSER\n$RAIDPASS"
                printf $LOGINSTR | $RAIDCOM -login | /dev/null
                $RAIDCFG -logint
                LOGGEDIN=$?
                [[ $LOGGEDIN -eq 0 ]] &&  echo "Logged into $SERIAL as $RAIDUSER"
                echo
        done
}

####################################################################################################
############### FUNCTION TO ASSIST IN REPORTING AT THE END##########################################
####################################################################################################

do_reports(){
	END_TIME=`date +%m%d%y%0k%0M`
	RPT_FILENAME=$SERIAL"_"$START_TIME
        touch $REPORT_DIR/$RPT_FILENAME
	echo "Start Time : $START_TIME" >> $REPORT_DIR/$RPT_FILENAME
	echo "End Time   : $END_TIME" >> $REPORT_DIR/$RPT_FILENAME
        
	echo "" >> $REPORT_DIR/$RPT_FILENAME
	echo "ZPR RUN BY $RAIDUSER USING $LDEVFILE ON $SERIAL" >> $REPORT_DIR/$RPT_FILENAME
        echo "" >> $REPORT_DIR/$RPT_FILENAME
       
	printf "%6s %10s %10s %10s \n" "LDEV" "TOTAL BLK" "USED BLK" "USED BLK"  >>  $REPORT_DIR/$RPT_FILENAME
        printf "%6s %10s %10s %10s \n" "LDEV" "ALLOCATED" "BEFORE ZPR" "AFTER ZPR"  >>  $REPORT_DIR/$RPT_FILENAME
        echo "=============================================================" >> $REPORT_DIR/$RPT_FILENAME

	for i in "${MASTER_LDEV_LIST[@]}"
	do
		 printf '%6s %10d %10d %10d\n' "$i" "${LDEV_TOTAL_CAPACITY[$i]}" "${START_LDEV_USED_CAPACITY[$i]}" "${END_LDEV_USED_CAPACITY[$i]}"\
			 >> $REPORT_DIR/$RPT_FILENAME
	done

        echo "" >> $REPORT_DIR/$RPT_FILENAME
	echo "SKIPPED LDEVS OWING TO LOCK ON ARRAY" >> $REPORT_DIR/$RPT_FILENAME
	
	for i in "${!SKIPPED_LDEVS_LOCK[@]}"
	do
		printf "%6s %6s \n" "$i" "${SKIPPED_LDEVS_LOCK[$i]}"  >> $REPORT_DIR/$RPT_FILENAME
	done

	
        echo "" >> $REPORT_DIR/$RPT_FILENAME
	echo "SKIPPED LDEVS OWING TO INVALID STATUS" >> $REPORT_DIR/$RPT_FILENAME
	
	for i in "${!SKIPPED_LDEVS_STATUS[@]}"
	do
		printf "%6s %6s \n" "$i" "${SKIPPED_LDEVS_STATUS[$i]}"  >> $REPORT_DIR/$RPT_FILENAME
	done

        echo "" >> $REPORT_DIR/$RPT_FILENAME
	echo "SKIPPED LDEVS OWING TO INVALID ATTRIBUTE" >> $REPORT_DIR/$RPT_FILENAME
	
	for i in "${!SKIPPED_LDEVS_INVALID[@]}"
	do
		printf "%6s %6s \n" "$i" "${SKIPPED_LDEVS_INVALID[$i]}"  >> $REPORT_DIR/$RPT_FILENAME
	done

} 

check_options(){
	[ -z $SERIAL ] && usage "Please enter Array Serial Number"
	[[ !( -z "$LDEVFILE" ) && !( -f "$LDEVFILE" ) ]] && usage "$LDEVFILE does not exist"
	[[ -z "$LDEVFILE" && -z "$LDEVLIST" ]] && usage "Enter either a LDEV filename or a list of LDEVs"
	printf "%5g" "$SERIAL" &> /dev/null && echo "" || usage "Invalid Serial Number"
}

print_banner(){

	echo "#####################################################################################################"
	echo "####################### ZERO PAGE RECLAIM UTILITY ######## v3.8  5.22.13   ##########################"
	echo "#####################################################################################################"
	echo
}

check_os_files(){

	UNAME=$(uname -a | grep CYGWIN)
	[[ $? -eq 0 ]] && ON_CYGWIN=0 || ON_CYGWIN=1
	if [ $ON_CYGWIN -eq 1 ]
	then
		RAIDCOM="/HORCM/etc/raidcom"
		HORCM="/HORCM/etc"
		HORCMROOT="/HORCM"
	fi

	if [[ ! ( -d "$REPORT_DIR" ) ]]
	then
		mkdir -p $REPORT_DIR
	fi

	if [ $DEBUG -eq 0 ]
	then
		[[ ( -f $RAIDCOM ) ]]  &&\
			 echo "Using $RAIDCOM version $VERSION" ||\
			 echo "Missing RAIDCOM executable" 
	fi
	
	[[ ( $DEBUG -eq 1 ) ]]  && RAIDCOM="./raidcom_cat.sh"

	if [ $DEBUG -eq 1 ]
	then
		[[ ( -f $RAIDCOM ) ]]  &&\
       		        echo "Running in offline mode. Using raidcom_cat.sh instead of /HORCM/usr/bin/raidcom" ||\
	    		echo "Missing raidcom_cat.sh" 
	fi
}

get_HORCMINST(){

	[[ ( $ON_CYGWIN -eq 1 ) ]] && HORCMINSTANCES=$(ps -ef | grep "horcmd_" | tr -s " " | cut -d" " -f8 | sed 's/horcmd_//' | tr "\n" " ")

	if [ $ON_CYGWIN -eq 0 ] 
	then
		cmd /c "wmic process where (CommandLine like '%horcmd_%') Get CommandLine" > /tmp/horcm.out
		dos2unix /tmp/horcm.out > /dev/null
		HORCMINSTANCES=$(cat /tmp/horcm.out | grep "^horcmd_" | sed 's/horcmd_//' | tr "\n" " ")
	fi

	local H_INST=""

	for i in $HORCMINSTANCES
	do
		$HORCM/raidqry -l -IH$i 2> /dev/null | grep $SERIAL > /dev/null
		if [ $? -eq 0 ]
		then 
 			H_INST=$i
		fi
	done		

	echo $H_INST
}


check_ldev_format(){

	local ldev=$1
	[[ "${ldev:0:1}" == "#" ]] && comment=0 || comment=1
	if [ $comment -eq 1 ]
	then
		ldev=${ldev//:/}
		len=${#ldev}
		[[ ( $len -lt 4 ) ]] && ldev=$(printf "%4s" $ldev | sed 's/ /0/g')
		[[ ( $len -eq 6 ) ]] && ldev=$(printf "%4s" ${ldev:2:4} | sed 's/ /0/g')
		printf $ldev | grep '^[-0-9a-zA-Z]*$' > /dev/null
		is_alph_num=$?
		[[ ( $is_alph_num -eq 0 ) ]] && ldev="0x"$ldev
	fi
	if [[ ( ${#ldev} -eq 6 ) && ( $comment -eq 1 ) ]]
	then
		echo $ldev
	else
		echo
	fi
}

get_total_blocks(){
	local TOTAL_BLOCKS=""
	TOTAL_BLOCKS=$($RAIDCOM get ldev -ldev_id $1 | grep "VOL_Capacity" | sed 's/VOL_Capacity(BLK)//' | sed 's/://g' | sed 's/ //g')
	echo $TOTAL_BLOCKS
}

get_used_blocks(){
	local USED_BLOCKS=""
	USED_BLOCKS=$($RAIDCOM get ldev -ldev_id $1 | grep "Used_Block" | sed 's/Used_Block(BLK)//' | sed 's/://g' | sed 's/ //g')
	echo $USED_BLOCKS
}

get_percent_used(){
	local PERCENT=0
	if [[ $1 -eq 0 ]]
	then
		PERCENT=0
	else
	        PERCENT=$( echo "$1/$2" | bc -l)
       		PERCENT=$( echo "$PERCENT * 100" | bc -l)
        	PERCENT=$(printf %.0f $PERCENT)
	fi
	echo $PERCENT
}

scan_for_threshold(){
	index=0
	count=0
	for ldev in "${MASTER_LDEV_LIST[@]}"
	do
		printf "."

		TOT_BLOCKS=$(get_total_blocks $ldev)

		USED_BLOCKS=$(get_used_blocks $ldev)
		PERCENT_USED=$(get_percent_used $USED_BLOCKS $TOT_BLOCKS)

	 	START_LDEV_USED_CAPACITY[$ldev]=$USED_BLOCKS
	 	LDEV_TOTAL_CAPACITY[$ldev]=$TOT_BLOCKS	
	 	END_LDEV_USED_CAPACITY[$ldev]=$USED_BLOCKS

		[[ $PERCENT_USED -lt $CUT_OFF_THRESHOLD ]] && CUT_OFF_LDEVS=("${CUT_OFF_LDEVS[@]}" $ldev) &&\
			MASTER_LDEV_LIST=(${MASTER_LDEV_LIST[@]:0:$index} ${MASTER_LDEV_LIST[@]:$(($index + 1))}) &&\
			(( index = $index -1 ))

		(( count = $count + 1 ))
		[[ $count -gt 79 ]] && printf "\n\t" 
		[[ $count -gt 79 ]] && (( count=0 )) 
		(( index = $index + 1 ))
	done
}

read_ldev_file(){
	index=0
	
	if [[ -n "$LDEVLIST" ]]
	then
		for i in $LDEVLIST
		do
			ldev=$(check_ldev_format $i)
			[[ -n "$ldev" ]] && MASTER_LDEV_LIST=("${MASTER_LDEV_LIST[@]}" $ldev) 
			[[ -z "$ldev" ]] && BAD_FORMAT_LDEV_LIST=("${BAD_FORMAT_LDEV_LIST[@]}" $i) 
		
			printf "."
			(( index = $index + 1 ))
			[[ $index -eq 80 ]] && printf "\n\t" && (( index=0 )) 
			
		done
	else
		if [[ ( -f "$LDEVFILE" ) ]]
		then
			dos2unix $LDEVFILE 2> /dev/null
			linecount=$(cat $LDEVFILE | grep -v "^#" | wc -l)
			for i in $(cat $LDEVFILE | grep -v "^#" | sed 's/^$//g' | tr "\n" " ")
			do
				ldev=$(check_ldev_format $i)
				[[ -n "$ldev" ]] && MASTER_LDEV_LIST=("${MASTER_LDEV_LIST[@]}" $ldev)
				[[ -z "$ldev" ]] && BAD_FORMAT_LDEV_LIST=("${BAD_FORMAT_LDEV_LIST[@]}" $i) 
	
				printf "."
				(( index = $index + 1 ))
				[[ $index -eq 80 ]] && printf "\n\t" && (( index=0 )) 
			done
			echo -e "\n\n\t\t$linecount lines in file $LDEVFILE"
		fi
	fi
}


filter_ldevs(){
	index=0	
	count=0
	VOL_TYPE=""
	check_result=""
	for ldev in "${MASTER_LDEV_LIST[@]}"
	do
		VOL_TYPE=$(check_for_TYPE $ldev)
		if [[ ( "$VOL_TYPE" == "OPEN-V" ) || ( "$VOL_TYPE" == "OPEN-V-CVS" ) ]]
		then
			check_result=$(check_for_ATTR $ldev)
			if [ "$check_result" == "HDP" ]
			then
				check_result=$(check_for_NML $ldev)
	
				if [ "$check_result" == "NML" ]
				then
					check_result=$(check_for_OPE $ldev)

					if [[ ( "$check_result" != "NONE" ) ]]
					then
						SKIPPED_LDEVS_STATUS[$ldev]=$check_result
						 MASTER_LDEV_LIST=(${MASTER_LDEV_LIST[@]:0:$index} ${MASTER_LDEV_LIST[@]:$(($index + 1))}) 
						(( index = $index -1 ))
						[[ $VERBOSE -eq 0 ]] && echo && echo "Will Skip ZPR on LDEV $ldev : Reason current operation is $check_result"
					fi
				else 
					SKIPPED_LDEVS_INVALID[$ldev]=$check_result
					 MASTER_LDEV_LIST=(${MASTER_LDEV_LIST[@]:0:$index} ${MASTER_LDEV_LIST[@]:$(($index + 1))}) 
					(( index = $index -1 ))
					[[ $VERBOSE -eq 0 ]] && echo && echo "Will Skip ZPR on LDEV $ldev : Reason status is $check_result"
				fi
			else
				SKIPPED_LDEVS_INVALID[$ldev]=$check_result
				 MASTER_LDEV_LIST=(${MASTER_LDEV_LIST[@]:0:$index} ${MASTER_LDEV_LIST[@]:$(($index + 1))}) 
				(( index = $index -1 ))
				[[ $VERBOSE -eq 0 ]] && echo && echo "Will Skip ZPR on LDEV $ldev : Reason attribute is $check_result"
			fi
		else
			SKIPPED_LDEVS_INVALID[$ldev]=$VOL_TYPE
			 MASTER_LDEV_LIST=(${MASTER_LDEV_LIST[@]:0:$index} ${MASTER_LDEV_LIST[@]:$(($index + 1))}) 
			(( index = $index -1 ))
			[[ $VERBOSE -eq 0 ]] && echo && echo "Will Skip ZPR on LDEV $ldev : Reason volume type is $VOL_TYPE"
		fi
		
		printf "."
		(( count = $count + 1 ))
		[[ $count -eq 80 ]] && printf "\n\t" 
		[[ $count -eq 80 ]] && (( count=0 )) 
		(( index = $index + 1 ))	
	done
}
				

run_zpr(){
	count=0	
	for ldev in "${MASTER_LDEV_LIST[@]}"
	do
		CURRENT_LDEV_LIST[$count]=$ldev
		(( count = $count + 1 ))
		
		if [ $count -eq $LDEVCOUNT ] 
			then
			        printf "\n\t\tStarting ZPR on LDEVs : "; echo ${CURRENT_LDEV_LIST[@]} 
				do_zpr_ldev ${CURRENT_LDEV_LIST[@]}
				[[ ( ( $? -eq 0 ) && ( $WAIT -eq 0 ) ) ]] && do_ldev_zpr_wait ${CURRENT_LDEV_LIST[@]}
				CURRENT_LDEV_LIST=()	
				count=0
			fi
	done
	if [ $count -gt 0 ]
	then
		printf "\n\t\tStarting ZPR on LDEVs : "; echo ${CURRENT_LDEV_LIST[@]} 
		LDEVCOUNT=$count
		do_zpr_ldev ${CURRENT_LDEV_LIST[@]} 
		[[ ( ( $? -eq 0 ) && ( $WAIT -eq 0 ) ) ]] && do_ldev_zpr_wait ${CURRENT_LDEV_LIST[@]}
	fi
}


##################################### START OF MAIN ###################################################################################

while getopts ws:c:f:l:v opt
do
	case "$opt" in
		w) WAIT=0;;
		s) SERIAL=$OPTARG;;
		c) LDEVCOUNT=$OPTARG;;
		f) LDEVFILE=$OPTARG;;
		l) LDEVLIST=$OPTARG;;
		v) VERBOSE=0;;
		\?) usage;;
		*) usage;;
	esac
done

check_options $#
	
print_banner

check_os_files

HORCMINST=$(get_HORCMINST)
if [[ -z "$HORCMINST" ]] 
then
	echo "Please start up HORCM instances" 
	exit 0
else
	echo "Using HORCMINST=$HORCMINST for $SERIAL"
fi

export HORCMINST

do_login	

printf "\nPHASE I  : Reading LDEV file and validating formats\n\t"

read_ldev_file

TOT_LDEVS=${#MASTER_LDEV_LIST[@]}

BAD_LDEVS=${#BAD_FORMAT_LDEV_LIST[@]}

[[ ( $TOT_LDEVS -eq 0 ) ]] && printf "\n\t\tNo valid LDEVs found." && exit 1
[[ ( $BAD_LDEVS -gt 0 ) ]] && printf "\n\t\tSkipped $BAD_LDEVS LDEVs in file in incorrect format\n" 
[[ ( $TOT_LDEVS -gt 0 ) ]] && printf "\n\t\t$TOT_LDEVS LDEVs will be processed\n" 

printf "\nPHASE II : Checking LDEV status and type\n\t"

[[ $VERBOSE -eq 0 ]] && echo && echo "Will check for LDEV States : ${VALID_LDEV_STATES[@]}"
[[ $VERBOSE -eq 0 ]] && echo "Valid LDEV States are : NML" && echo

[[ $VERBOSE -eq 0 ]] && echo "Will check for Operating Values : ${VALID_OPE_VALUES[@]}"
[[ $VERBOSE -eq 0 ]] && echo "Valid OPE Values are : NONE ZPD" && echo 

[[ $VERBOSE -eq 0 ]] && echo "Will check for Volume Attributes : ${VALID_VOL_ATTR[@]}"
[[ $VERBOSE -eq 0 ]] && echo "Valid Volume Attributes are : CVS HDP HDT" && echo

filter_ldevs

printf "\n"	


TOT_SKIP_ST_LDEVS=${#SKIPPED_LDEVS_STATUS[@]}


TOT_SKIP_IN_LDEVS=${#SKIPPED_LDEVS_INVALID[@]}

TEMP_LDEV_LIST=()

if [[ $TOT_SKIP_ST_LDEVS -gt 0 ]]
then
	echo -e "\n\t\tSkipped $TOT_SKIP_ST_LDEVS LDEVs owing to invalid operation status\n"

	for i in "${!SKIPPED_LDEVS_STATUS[@]}"
	do
		STATUS_STR=${SKIPPED_LDEVS_STATUS[$i]}
	
		STATUS_STR=( ${STATUS_STR//[,\/]/ } )
		for sts in ${STATUS_STR[@]}
		do
			 check_list_item $sts ${SKIP_VOL_OPE_TYPE[@]}
			 if [[ $? -eq 0 ]]
			 then
				if [[ -z "${TEMP_LDEV_LIST[$sts]}" ]]
				then
					TEMP_LDEV_LIST[$sts]=$i
				else
					TEMP_LDEV_LIST[$sts]=${TEMP_LDEV_LIST[$sts]}" "$i
				fi
			 fi
		done
	done
	
	for i in "${!TEMP_LDEV_LIST[@]}"
	do
		printf "\t\t $i : ${TEMP_LDEV_LIST[$i]} \n"
	done
fi

TEMP_LDEV_LIST=()

if [[ $TOT_SKIP_IN_LDEVS -gt 0 ]]
then
        echo -e "\n\t\tSkipped $TOT_SKIP_IN_LDEVS LDEVs owing to invalid attribute or state\n"

        for i in "${!SKIPPED_LDEVS_INVALID[@]}"
        do
                STATUS_STR=${SKIPPED_LDEVS_INVALID[$i]}

                STATUS_STR=( ${STATUS_STR//[,\/]/ } )
                for sts in ${STATUS_STR[@]}
                do
                         check_list_item $sts ${SKIP_VOL_ATTR[@]}
                         if [[ $? -eq 0 ]]
                         then
                                if [[ -z "${TEMP_LDEV_LIST[$sts]}" ]]
                                then
                                        TEMP_LDEV_LIST[$sts]=$i
                                else
                                        TEMP_LDEV_LIST[$sts]=${TEMP_LDEV_LIST[$sts]}" "$i
                                fi
                         fi
                done
        done

        for i in "${!TEMP_LDEV_LIST[@]}"
        do
                printf "\t\t $i : ${TEMP_LDEV_LIST[$i]} \n"
        done
fi


TOT_LDEVS=${#MASTER_LDEV_LIST[@]}
[[ ( $TOT_LDEVS -gt 0 ) ]] && printf "\n\t\t$TOT_LDEVS LDEVs will be processed \n" 

printf "\nPHASE III : Zero Page Reclaim\n"

TOT_LDEVS=${#MASTER_LDEV_LIST[@]}

[[ ( $TOT_LDEVS -eq 0 ) ]] && printf "\n\t\tNo valid LDEVs found." && exit 1

printf "\n\t\tScanning for cut off thresholds\n\t"

scan_for_threshold

printf "\n"	

CUT_LDEVS=${#CUT_OFF_LDEVS[@]}
TOT_LDEVS=${#MASTER_LDEV_LIST[@]}

[[ ( $TOT_LDEVS -eq 0 ) ]] && printf "\n\t\tNo valid LDEVs found." && exit 1
[[ ( $CUT_LDEVS -gt 0 ) ]] && printf "\n\t\tSkipped $CUT_LDEVS LDEVs owing to being below $CUT_OFF_THRESHOLD%% Used threshold\n" 

cnt=0

if [[ $CUT_LDEVS -gt 0 ]]
then
	printf "\t"
	for i in "${CUT_OFF_LDEVS[@]}"
	do
		[[ $cnt -eq $LDEVCOUNT ]] && printf "\n\t"
		[[ $cnt -eq $LDEVCOUNT ]] && (( cnt=0 ))
		printf "\t $i:"
		PERCENT_USED=$(get_percent_used  ${START_LDEV_USED_CAPACITY[$i]} ${LDEV_TOTAL_CAPACITY[$i])})
		printf ":%3d%% " $PERCENT_USED 
		[[ $cnt -lt $LDEVCOUNT ]] &&  (( cnt=$cnt + 1 ))

	done
fi

for i in "${MASTER_LDEV_LIST[@]}"
do
    (( PROCESSED_LDEVS[$i]=1 ))
    ((  COUNT_LDEV_ZPR_CHECK[$i]=0 ))
done


[[ ( $TOT_LDEVS -gt 0 ) ]] && printf "\n\t\t$TOT_LDEVS LDEVs will be processed \n" 

run_zpr

$RAIDCOM -logout

do_reports


##################################### END OF MAIN #####################################################################################
